;===========================;
; Hammer, From Paper Mario  ;
;===========================;===================;
;						;
;Description: A Hammer sprite similar to	;
;The one in Paper Mario.			;
; Some code taken from hammer.asm		;
;===============================================;

!Tile = $E4	; tile on ground

HammerTiles:           	                   
	;db $C0,$C0,$CE,$88,$CE,$88,$CE,$C0
	db $E4,$E4,$E6,$E8,$E6,$E8,$E6,$E4

; ^ Tiles for the hammer ^

!SprGfxRt1x1 = $0190B2

print "INIT ",pc
	;LDA $E4,x	;\
	;LSR #4		; |
	;AND #$01	; |
	;STA $1594,x	;/ X Pos*2.
        RTL

;==================;
; Sprite Main JSL  ;
;==================;

print "MAIN ",pc
	PHB		;\
        PHK		; |
        PLB		; | Change data bank to our one.
	JSR Spr		; | Sprite main routine.
	JSR Decr	; | Control $163E,x.
	PLB		; | Pull old data bank ..
	RTL		;/

Decr:
	LDA $163E,x	;\
	BEQ +		; | Don't allow $163E,x to become negative.
	DEC $163E,x	;/
+	RTS

;==========================================================================
; Sprite Main Routine
;==========================================================================

Spr:
	LDA $C2,x	;\ Use $C2,x ..
	ASL A		; |
	PHX		; | Preserve X.
	TAX		; | As a pointer.
	JMP (Ptr,x)	;/
Ptr:
	dw HammerOnGround ; 00 ; On the ground.
	dw MarioHasHammer ; 01 ; Has hammer.
	dw ApplyHammer	  ; 02 ; Apply hammer.
;==========================================================================
; Hammer On Ground (00)
;==========================================================================

HammerOnGround:
	PLX			; Pull back sprite index.
	JSR SubOffScreen	; Handle sprite offscreen.

	JSL !SprGfxRt1x1	;\
	LDY $15EA,x		; |
	;LDA HammerTiles+1	; | Draw sprite on ground.
	LDA #!Tile
	STA $0302,y		;/

	;LDA $1594,x		;\ Check to see if Mario has Hammer "B"..
	;BNE StaticVoid		; |
	;LDA #$00		; | If so, load palette 9 into $15F6,x.
	;BRA DoneWithPal
;StaticVoid:
;	LDA #$02
;DoneWithPal:

	LDA #$01
	STA $15F6,x
	LDA $187A		; If Mario is on Yoshi ..
	BNE NoContact		; ... skip.
	JSL $01A7DC		; If Mario isn't touching the sprite ..
	BCC NoContact		; ... skip.

	LDA #$01		;\
	STA $C2,x		; | Status = has hammer.
	STA $19			;/ Make big.
	LDA #$04		;\
	STA $71			;/ Set animation.
	LDA #$30		;\
	STA $149B		;/ .. and animation timer.
	LDA #$0A		;\
	STA $1DF9		;/ Play sound effect.
NoContact:
	RTS
;==========================================================================
; Mario Has Hammer (01)
;==========================================================================

MarioHasHammer:
	PLX			; Restore sprite index.
	LDA $71			;\
	CMP #$09		; | If Mario is dead ..
	BNE Normal		; |
	STZ $14C8,x		;/ Erase the sprite.
	RTS
Normal:
	LDA $16			;\
	AND #$40		; | Check to see if pressing the required key.
	BEQ Skip		;/ If not, bypass.
	LDA $163E,x		;\ IF the timer for images != 0 ..
	ORA $73			; | Or ducking ..
	ORA $187A		; | Or on Yoshi ..
	ORA $75			; | Or in water ..
	BNE Skip		;/ Bypass routine.

	LDA $77			;\ Don't function ..
	AND #$04		; | If in air.
	BEQ Skip		;/

	LDA #$02		;\
	STA $C2,x		;/ Status = applying hammer.
	LDA #$3C		;\
	STA $163E,x		;/ Set duration.
Skip:
	RTS
;==========================================================================
; Applying Hammer (02)
;==========================================================================

MarioImages:
	db $08,$08,$08,$08,$08,$08,$08,$09 ; Poses.
	;db $35,$36,$36,$37,$37,$36,$36,$35	
DispX:	               	                   
	db $FF,$FF,$0A,$0B,$0B,$0B,$0B,$06 ; XDISP.
DispXHi:               	                   
	db $FF,$FF,$00,$00,$00,$00,$00,$00 ; XDISP High.
DispY:
	db $F0,$FA,$02,$06,$10,$06,$02,$FC ;\
	db $FB,$F6,$FD,$06,$10,$06,$FD,$FF ;/ YDISP.
DispYHi:               	                   
	db $00,$FF,$00,$00,$00,$00,$00,$FF ;\
	db $FF,$FF,$FF,$00,$00,$00,$FF,$FF ;/ Y DISP High.

Finish:
	STZ $C2,x		; No hammer.
	STZ $1528,x		; Don't show it.
	STZ $14C8,x		; Sprite finished!
	RTS

TimeUp:
	LDA #$01		;\
	STA $C2,x		; | Revert to has hammer.
	STZ $1528,x		; | Branches if timer is up.
	RTS			;/

ApplyHammer:
	PLX			; Restore sprite index.
	LDA $163E,x		;\
	BEQ TimeUp		;/ If time is over, revert to 01.

	LDA $71
	CMP #$01		; If hurt, just end.
	BEQ Finish
	LDA $19
	CMP #$01
	BNE Finish

	LDA #$FF		;\
	STA $0DAA		; | Don't allow Mario to jump/spin-jump while he swings the hammer.
	STA $0DAC		;/
Stored:
	LDA $14			;\
	INC A			; |
	LSR #2			; |
	AND #$07		; | Switch between frames.
	TAY			; |
	LDA MarioImages,y	; | Load poses..
	STA $13E0		;/ Store to $13E0.

	LDA #$02		;\ Set carrying item flag.
	STA $148F		;/
	
	INC $1528,x		;/ Increase number of frames using hammer ..
	LDA $1528,x		;\
	LSR #2			; | Use $1528,x as an animation counter.
	AND #$01		; | 7 frames ..
	STA $03			;/ Into $03.

	PHX			;/ Preserve sprite index.
	LDX $19			;\ IF Mario status = small ..
	BEQ NotSmall		; |
	CLC			; | Y displacement = 08.
	ADC #$08		;/
NotSmall:
	PLX			; Pull sprite index.
	TAY			; $03 -> Y.
	LDA $D3			;\
	CLC			; |
	ADC DispY,y		; | Apply Y displacement.
	STA $D8,x		;/ To sprite low position.
	LDA $D4			;\
	ADC DispYHi,y		; | Apply displacement on high byte if $D3 overflows.
	STA $14D4,x		;/ Store to sprite high byte.
	
	LDY $03			;\ Use $03 as an index.
	LDA $76			; | If facing left ..
	BEQ Subtract		;/ Subtract displacement.

	LDA $94			;\
	CLC			; |
	ADC DispX,y		; | Apply X diplacement.
	;INC			; |
	STA $E4,x		;/ .. to sprite X position low byte.
	LDA $95			;\
	ADC DispXHi,y		; | Add to high byte so $94 doesn't overflow.
	STA $14E0,x		;/
	BRA SkipSpr
Subtract:
	LDA $94			;\
	SEC			; |
	SBC DispX,y		; | Apply X diplacement.
	DEC
	STA $E4,x		;/ .. to sprite X position low byte.
	LDA $95			;\
	SBC DispXHi,y		; | Add to high byte so $94 doesn't overflow.
	STA $14E0,x		;/
SkipSpr:
	JSR Tilemaps		;/ Draw tilemaps.
	LDA $9D			;\ If sprites are locked, return.
	BNE Return		;/
	JSR Interact		;\ Interact with sprites.
Return: RTS

;==========================================================================
; Interaction With Sprites
;==========================================================================
Interact:
	LDA $77
	AND #$04
	BEQ Bypass
	LDA $1528,x
	CMP #$13
	BNE Bypass
	LDA $1594,x
	BEQ +
	JSL $0294C1
	JSL $0286BF
+
	LDY $1594,x
	LDA Timer,y
	STA $1887
	LDA #$09		;\
	STA $1DFC		;/ Sound.
Bypass:
	LDY #$0B		; Y = index to number of loops.
LoopS:
	LDA $14C8,y		;\
	CMP #$08		; | Skip dead sprites.
	BCC NextSpr		;/
	LDA $1686,y		;\
	AND #$08		; | Skip sprites that don't interact with others.
	BNE NextSpr		;/
	JSR SprInteract		;\ Otherwise interact.
NextSpr:
	DEY			;\ Decrease sprite slots ..
	BPL LoopS		;/ Loop again.
	RTS

SprInteract:
	JSL $03B69F		;\ Check sprite clipping A ..
	PHX			; | Push sprite index.
	TYX			;/ Our sprite -> X.
	JSL $03B6E5		;\ Check sprite clipping B..
	PLX
	JSL $03B72B		;/ Check for contact ..	
	BCC NoCont		;\ Return if not any.
	PHX			; |
	TYX			; |
	LDA #$02		; |
	STA $14C8,x		;/ Kill sprite.
	LDA #$D0		;\
	STA $AA,x		;/ Make it fall.
	JSL $01AB72		;\ Apply contact tilemap.
	PLX			;/ Pull back old sprite index.

	LDA #$03		;\
	STA $1DF9		;/ Play sound.
	LDA #$01		;\
	JSL $02ACE5		;/ And give points.
NoCont:
	RTS
Timer: db $10,$18
;==========================================================================
; Sprite Tilemap
;==========================================================================

Direction:             	                   
	db $40,$00,$00,$00,$80,$00,$00,$00
Flip:
	db $40,$00	
Tilemaps:
	JSR GetDrawInfo	
	PHX			; Push sprite index.
	
	LDX $76			;\
	LDA Flip,x		; | Flip if direction = left.
	STA $02			;/ Store into $02.

	LDX $03			; Use $03 as X index.
	
        LDA $00         	;\      
        STA $0300,y           	;/ Store X position of sprite.
        
        LDA $01   		;\            
        STA $0301,y           	;/ Store Y position of sprite.

	LDA HammerTiles,x	;\
	STA $0302,y		;/ Store tiles based on $03.

;YXPPCCCT
;00000001

	PHX
	LDX $15E9
	LDA $1594,x		;\ If Mario has old hammer ..
	BEQ GetInfoA		; | Branch to 00 $15F6,x.
	LDA #$02
	STA $15F6,x		; Grey hammer.
	BRA DoInfo
GetInfoA:
	LDA #$01
	STA $15F6,x
DoInfo:
        LDA $15F6,x 		;/ Load palette info (CCCT).
        PLX			;\ Pull back $03.
        ORA $02			;/ Add in "flip" byte.
        EOR Direction,x		;\ Apply inversion on direction.
        ORA $64			;/ Add priority byte.
        STA $0303,y		;\ Store to YXPPCCCT byte.
	
        PLX     		;/ Retrieve sprite index.
        LDY #$02		;\ 16x16 tiles.
        LDA #$01		; | We drew 2 tiles.	
        JSL $01B7B3		;/ Call OAM write.
NoDraw:	
        RTS

;==========================================================================
; Borrowed Routines
;==========================================================================

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; $B85D - off screen processing code - shared
; sprites enter at different points
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

TABLE3:              db $40,$B0
TABLE6:              db $01,$FF 
TABLE4:              db $30,$C0,$A0,$80,$A0,$40,$60,$B0 
TABLE5:              db $01,$FF,$01,$FF,$01,$00,$01,$FF

SubOffScreen:       JSR SUB_IS_OFF_SCREEN   ; \ if sprite is not off screen, return
                    BEQ RETURN_2            ; /    
                    LDA $5B                 ; \  goto VERTICAL_LEVEL if vertical level
                    AND #$01                ;  |
                    BNE VERTICAL_LEVEL      ; /     
                    LDA $D8,x               ; \
                    CLC                     ;  | 
                    ADC #$50                ;  | if the sprite has gone off the bottom of the level...
                    LDA $14D4,x             ;  | (if adding 0x50 to the sprite y position would make the high byte >= 2)
                    ADC #$00                ;  | 
                    CMP #$02                ;  | 
                    BPL ERASE_SPRITE        ; /    ...erase the sprite
                    LDA $167A,x             ; \ if "process offscreen" flag is set, return
                    AND #$04                ;  |
                    BNE RETURN_2            ; /
                    LDA $13                 ; \ 
                    AND #$01                ;  | 
                    STA $01                 ;  |
                    TAY                     ; /
                    LDA $1A                 ;x boundry 
                    CLC                     
                    ADC TABLE4,y            
                    ROL $00                 
                    CMP $E4,x               ;x pos 
                    PHP                     
                    LDA $1B                 ;x boundry hi 
                    LSR $00                 
                    ADC TABLE5,y            
                    PLP                     
                    SBC $14E0,x             ;x pos high 
                    STA $00                 
                    LSR $01                 
                    BCC LABEL20             
                    EOR #$80                
                    STA $00                 
LABEL20:             LDA $00                 
                    BPL RETURN_2            
ERASE_SPRITE:        LDA $14C8,x             ; \ if sprite status < 8, permanently erase sprite
                    CMP #$08                ;  |
                    BCC KILL_SPRITE         ; /
                    LDY $161A,x             
                    CPY #$FF                
                    BEQ KILL_SPRITE         
                    LDA #$00                ; \ mark sprite to come back   
                    STA $1938,y             ; /     
KILL_SPRITE:         STZ $14C8,x             ; erase sprite
RETURN_2:            RTS                     ; return

VERTICAL_LEVEL:      LDA $167A,x             ; \ if "process offscreen" flag is set, return
                    AND #$04                ;  |
                    BNE RETURN_2            ; /
                    LDA $13                 ; \ only handle every other frame??
                    LSR A                   ;  | 
                    BCS RETURN_2            ; /
                    AND #$01                
                    STA $01                 
                    TAY                     
                    LDA $1C                 
                    CLC                     
                    ADC TABLE3,y            
                    ROL $00                 
                    CMP $D8,x               
                    PHP                     
                    LDA $001D             
                    LSR $00                 
                    ADC TABLE6,y            
                    PLP                     
                    SBC $14D4,x             
                    STA $00                 
                    LDY $01                 
                    BEQ LABEL22             
                    EOR #$80                
                    STA $00                 
LABEL22:             LDA $00                 
                    BPL RETURN_2            
                    BMI ERASE_SPRITE        

SUB_IS_OFF_SCREEN:   LDA $15A0,x             ; \ if sprite is on screen, accumulator = 0 
                    ORA $186C,x             ;  |  
                    RTS                     ; / return

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; GET_DRAW_INFO
; This is a helper for the graphics routine.  It sets off screen flags, and sets up
; variables.  It will return with the following:
;
;		Y = index to sprite OAM ($300)
;		$00 = sprite x position relative to screen boarder
;		$01 = sprite y position relative to screen boarder  
;
; It is adapted from the subroutine at $03B760
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 	
;IF YOU FIND THIS, YOU WIN A COOKIE!
;lol

DATA_03B75C:        db $0C,$1C
DATA_03B75E:        db $01,$02

GetDrawInfo:        STZ $186C,X               ; Reset sprite offscreen flag, vertical 
                    STZ $15A0,X               ; Reset sprite offscreen flag, horizontal 
                    LDA $E4,X                 ; \ 
                    CMP $1A                   ;  | Set horizontal offscreen if necessary 
                    LDA $14E0,X               ;  | 
                    SBC $1B                   ;  | 
                    BEQ ADDR_03B774           ;  | 
                    INC $15A0,X               ; / 
ADDR_03B774:        LDA $14E0,X               ; \ 
                    XBA                       ;  | Mark sprite invalid if far enough off screen 
                    LDA $E4,X                 ;  | 
                    REP #$20                  ; Accum (16 bit) 
                    SEC                       ;  | 
                    SBC $1A                   ;  | 
                    CLC                       ;  | 
                    ADC #$0040              ;  | 
                    CMP #$0180              ;  | 
                    SEP #$20                  ; Accum (8 bit) 
                    ROL                       ;  | 
                    AND #$01                ;  | 
                    STA $15C4,X               ;  | 
                    BNE ADDR_03B7CF           ; /  
                    LDY #$00                ; \ set up loop: 
                    LDA $1662,X               ;  |  
                    AND #$20                ;  | if not smushed (1662 & 0x20), go through loop twice 
                    BEQ ADDR_03B79A           ;  | else, go through loop once 
                    INY                       ; /                        
ADDR_03B79A:        LDA $D8,X                 ; \                        
                    CLC                       ;  | set vertical offscree 
                    ADC DATA_03B75C,Y         ;  |                       
                    PHP                       ;  |                       
                    CMP $1C                   ;  | (vert screen boundry) 
                    ROL $00                   ;  |                       
                    PLP                       ;  |                       
                    LDA $14D4,X               ;  |                       
                    ADC #$00                ;  |                       
                    LSR $00                   ;  |                       
                    SBC $1D                   ;  |                       
                    BEQ ADDR_03B7BA           ;  |                       
                    LDA $186C,X               ;  | (vert offscreen)      
                    ORA DATA_03B75E,Y         ;  |                       
                    STA $186C,X               ;  |                       
ADDR_03B7BA:        DEY                       ;  |                       
                    BPL ADDR_03B79A           ; /                        
                    LDY $15EA,X               ; get offset to sprite OAM                           
                    LDA $E4,X                 ; \ 
                    SEC                       ;  |                                                     
                    SBC $1A                   ;  |                                                    
                    STA $00                   ; / $00 = sprite x position relative to screen boarder 
                    LDA $D8,X                 ; \                                                     
                    SEC                       ;  |                                                     
                    SBC $1C                   ;  |                                                    
                    STA $01                   ; / $01 = sprite y position relative to screen boarder 
                    RTS                       ; Return 

ADDR_03B7CF:        PLA                       ; \ Return from *main gfx routine* subroutine... 
                    PLA                       ;  |    ...(not just this subroutine) 
                    RTS                       ; / 